package frame

import (
	"encoding/json"
	"fmt"

	"git.orcatech.org/sdks/golang/monitor"
)

/* This is generated code */

// Frame ...
type Frame struct {
	monitor.AlertFrame
	monitor.LogFrame

	GetBulbsRequest  *GetBulbsRequest  `json:"requestGetBulbs,omitempty"`
	GetBulbsResponse *GetBulbsResponse `json:"responseGetBulbs,omitempty"`

	PermitJoiningRequest  *PermitJoiningRequest  `json:"requestPermitJoining,omitempty"`
	PermitJoiningResponse *PermitJoiningResponse `json:"responsePermitJoining,omitempty"`

	StopPermitJoiningRequest  *StopPermitJoiningRequest  `json:"requestStopPermitJoining,omitempty"`
	StopPermitJoiningResponse *StopPermitJoiningResponse `json:"responseStopPermitJoining,omitempty"`

	TurnOnLightRequest  *TurnOnLightRequest  `json:"requestTurnOnLight,omitempty"`
	TurnOnLightResponse *TurnOnLightResponse `json:"responseTurnOnLight,omitempty"`

	TurnOffLightRequest  *TurnOffLightRequest  `json:"requestTurnOffLight,omitempty"`
	TurnOffLightResponse *TurnOffLightResponse `json:"responseTurnOffLight,omitempty"`

	ChangeBrightnessRequest  *ChangeBrightnessRequest  `json:"requestChangeBrightness,omitempty"`
	ChangeBrightnessResponse *ChangeBrightnessResponse `json:"responseChangeBrightness,omitempty"`

	ChangeColorTemperatureRequest  *ChangeColorTemperatureRequest  `json:"requestChangeColorTemperature,omitempty"`
	ChangeColorTemperatureResponse *ChangeColorTemperatureResponse `json:"responseChangeColorTemperature,omitempty"`

	BulbsChangedSubscription *BulbsChangedSubscription `json:"subscriptionBulbsChanged,omitempty"`

	FirmwareUpdateStepSubscription *FirmwareUpdateStepSubscription `json:"subscriptionFirmwareUpdateStep,omitempty"`

	DongleStatusSubscription *DongleStatusSubscription `json:"subscriptionDongleStatus,omitempty"`
}

// New ...
func (p *Frame) New() interface{} {
	return &Frame{}
}

func (f *Frame) TopicMQTT() string {
	return ""
}

func (p *Frame) String() string {
	b, err := json.Marshal(p)
	if err != nil {
		return err.Error()
	}
	return string(b)
}

// Validate ...
func (p *Frame) Validate() error {
	var err error
	if p.GetBulbsRequest != nil {
		if err = p.GetBulbsRequest.Validate(); err != nil {
			return err
		}
	}

	if p.GetBulbsResponse != nil {
		if err = p.GetBulbsResponse.Validate(); err != nil {
			return err
		}
	}
	if p.PermitJoiningRequest != nil {
		if err = p.PermitJoiningRequest.Validate(); err != nil {
			return err
		}
	}

	if p.PermitJoiningResponse != nil {
		if err = p.PermitJoiningResponse.Validate(); err != nil {
			return err
		}
	}
	if p.StopPermitJoiningRequest != nil {
		if err = p.StopPermitJoiningRequest.Validate(); err != nil {
			return err
		}
	}

	if p.StopPermitJoiningResponse != nil {
		if err = p.StopPermitJoiningResponse.Validate(); err != nil {
			return err
		}
	}
	if p.TurnOnLightRequest != nil {
		if err = p.TurnOnLightRequest.Validate(); err != nil {
			return err
		}
	}

	if p.TurnOnLightResponse != nil {
		if err = p.TurnOnLightResponse.Validate(); err != nil {
			return err
		}
	}
	if p.TurnOffLightRequest != nil {
		if err = p.TurnOffLightRequest.Validate(); err != nil {
			return err
		}
	}

	if p.TurnOffLightResponse != nil {
		if err = p.TurnOffLightResponse.Validate(); err != nil {
			return err
		}
	}
	if p.ChangeBrightnessRequest != nil {
		if err = p.ChangeBrightnessRequest.Validate(); err != nil {
			return err
		}
	}

	if p.ChangeBrightnessResponse != nil {
		if err = p.ChangeBrightnessResponse.Validate(); err != nil {
			return err
		}
	}
	if p.ChangeColorTemperatureRequest != nil {
		if err = p.ChangeColorTemperatureRequest.Validate(); err != nil {
			return err
		}
	}

	if p.ChangeColorTemperatureResponse != nil {
		if err = p.ChangeColorTemperatureResponse.Validate(); err != nil {
			return err
		}
	}
	if p.BulbsChangedSubscription != nil {
		if err = p.BulbsChangedSubscription.Validate(); err != nil {
			return err
		}
	}
	if p.FirmwareUpdateStepSubscription != nil {
		if err = p.FirmwareUpdateStepSubscription.Validate(); err != nil {
			return err
		}
	}
	if p.DongleStatusSubscription != nil {
		if err = p.DongleStatusSubscription.Validate(); err != nil {
			return err
		}
	}
	return err
}

// NewInvalidFieldError ...
func NewInvalidFieldError(value string, fieldName string) error {
	return fmt.Errorf("Invalid value for %s: %s", fieldName, value)
}
