import { IPC } from 'orcatech-ipc';
import { AlertsIPC } from 'orcatech-ipc-alerts';
import Subscription from './subscriptions_gen';
import Command from './commands_gen';
import Frame from './frame/frame_gen';

export default class HueIPC extends IPC {

	constructor(host, port) {
		super(host, port, Frame);
		this.alerts = new AlertsIPC(this);
	}

	getBulbs = (onSuccess, onError) => {
		let onDecompose = (handler, msg) => {
			handler(msg.responseGetBulbs.bulbs);
		};
		let f = new Frame();
		f.requestGetBulbs.must = true;
		return this.write(Command.getBulbs, f, onSuccess, onError, onDecompose);
	}

	permitJoining = (onSuccess, onError) => {
		let f = new Frame();
		f.requestPermitJoining.must = true;
		return this.write(Command.permitJoining, f, onSuccess, onError);
	}

	stopPermitJoining = (onSuccess, onError) => {
		let f = new Frame();
		f.requestStopPermitJoining.must = true;
		return this.write(Command.stopPermitJoining, f, onSuccess, onError);
	}

	turnOnLight = (eui, onSuccess, onError) => {
		let f = new Frame();
		f.requestTurnOnLight.must = true;
		f.requestTurnOnLight.eui = eui;
		return this.write(Command.turnOnLight, f, onSuccess, onError);
	}

	turnOffLight = (eui, onSuccess, onError) => {
		let f = new Frame();
		f.requestTurnOffLight.must = true;
		f.requestTurnOffLight.eui = eui;
		return this.write(Command.turnOffLight, f, onSuccess, onError);
	}

	changeBrightness = (eui, brightness, onSuccess, onError) => {
		let f = new Frame();
		f.requestChangeBrightness.must = true;
		f.requestChangeBrightness.eui = eui;
		f.requestChangeBrightness.brightness = brightness;
		return this.write(Command.changeBrightness, f, onSuccess, onError);
	}

	changeColorTemperature = (eui, temperature, onSuccess, onError) => {
		let f = new Frame();
		f.requestChangeColorTemperature.must = true;
		f.requestChangeColorTemperature.eui = eui;
		f.requestChangeColorTemperature.temperature = temperature;
		return this.write(Command.changeColorTemperature, f, onSuccess, onError);
	}

	unsubscribeBulbsChanged = (onSuccess, onError) => {
		return this.unsubscribe(Subscription.bulbsChanged, onSuccess, onError);
	}

	subscribeBulbsChanged = (onBulbsChanged, onSuccess, onError) => {
		let onDecompose = (handler, msg) => {
			handler(msg.subscriptionBulbsChanged.bulbs);
		};
		return this.subscribe(Subscription.bulbsChanged, onBulbsChanged, onSuccess, onError, onDecompose);
	}

	unsubscribeFirmwareUpdateStep = (onSuccess, onError) => {
		return this.unsubscribe(Subscription.firmwareUpdateStep, onSuccess, onError);
	}

	subscribeFirmwareUpdateStep = (onFirmwareUpdateStep, onSuccess, onError) => {
		let onDecompose = (handler, msg) => {
			handler(msg.subscriptionFirmwareUpdateStep.percentageComplete);
		};
		return this.subscribe(Subscription.firmwareUpdateStep, onFirmwareUpdateStep, onSuccess, onError, onDecompose);
	}

	unsubscribeDongleStatus = (onSuccess, onError) => {
		return this.unsubscribe(Subscription.dongleStatus, onSuccess, onError);
	}

	subscribeDongleStatus = (onDongleStatus, onSuccess, onError) => {
		let onDecompose = (handler, msg) => {
			handler(msg.subscriptionDongleStatus.connected);
		};
		return this.subscribe(Subscription.dongleStatus, onDongleStatus, onSuccess, onError, onDecompose);
	}
}
